module Main where

import Control.Monad
import Data.Char
import Data.Function
import Data.List
import Data.Maybe
import System.Directory
import System.Environment
import System.FilePath
import System.IO
import Text.ParserCombinators.Parsec
import Text.Printf

data Result = Result {
    planner :: String,
    problem :: String,
    pfile :: String,
    size :: Int,
    finished :: Bool,
    realTime :: Double,
    userTime :: Double,
    sysTime :: Double
} deriving Eq

cpuTime r = userTime r + sysTime r

instance Show Result where
    show r = printf "%s %s %d %s %f %f %f"
        (planner r)
        (problem r)
        (size r)
        (show $ finished r)
        (realTime r)
        (userTime r)
        (sysTime r)

instance Ord Result where
    compare r1 r2 =
        let comparisons = filter (/= EQ) [
                compare (problem r1) (problem r2),
                compare (planner r1) (planner r2)
                ]
        in
        head $ comparisons ++ [EQ]

defaultResult = Result {
    problem = undefined,
    planner = undefined,
    pfile = undefined,
    size = undefined,
    finished = False,
    realTime = 2e32, -- 1/0,
    userTime = 2e32,
    sysTime = 2e32
}

resultHeader = "Variant Problem Size Finished Real User System"


parseTimes runID template contents =
    let Right times = runParser (many timeParser) () (runID ++ planner template) contents in
    times
    where
        timeParser = do
            r <- nameP 
            changes <- try (string "FAIL" >> spaces >> return [id]) 
                <|> many (try timeMeasure)
            return $ foldl (\r' f -> f r') r changes
        timeMeasure =
            (string "real" >> spaces >> doubleP >>= \x -> return $ \r' -> r' {realTime = x, finished = True})
            <|> (string "user" >> spaces >> doubleP >>= \x -> return $ \r' -> r' {userTime = x, finished = True})
            <|> (string "sys" >> spaces >> doubleP >>= \x -> return $ \r' -> r' {sysTime = x, finished = True})
        doubleP = do
            i <- many digit
            char '.'
            d <- many digit
            spaces
            return $ read $ i ++ "." ++ d
        nameP = do
            string "pfile_"
            i <- many digit
            rest <- manyTill anyChar space
            spaces
            return $ template { 
                problem = runID ++ "_" ++ i, 
                pfile = "pfile_" ++ i ++ rest,
                size = read i }

printResults dir = do
    ff <- liftM (parseTimes dir (defaultResult { planner = "FF-Plain" })) $
        readFile $ joinPath [dir, "times.noh"]
    ffh <- liftM (parseTimes dir (defaultResult { planner = "FF-HTN" })) $
        readFile $ joinPath [dir, "times.htn"]
    mapM_ print $
        ff ++ ffh
    
main = do
    directories <- getArgs
    --results <- foldM collectResults [] directories
    putStrLn resultHeader
    --mapM_ (putStrLn . show) results
    mapM_ printResults directories
